import mprofile
import threading
import time
import unittest


def alloc_objects():
    l = []
    for i in range(1000):
        l.append(object())
        time.sleep(0.001)
    return l


class TestMProfiler(unittest.TestCase):
    def test_profile_one(self):
        mprofile.start()
        alloc_obj = object()
        snap = mprofile.take_snapshot()
        mprofile.stop()

        self.assertEqual(len(snap.traces), 1)
        frame = snap.traces[0].traceback[-1]
        self.assertTrue(frame.filename.endswith("test_mprofile.py"))
        self.assertEqual(frame.name, "test_profile_one")
        self.assertEqual(frame.firstlineno, 16)
        self.assertEqual(frame.lineno, 18)

    def test_profile(self):
        mprofile.start()
        large_alloc = [object() for _ in range(12)]
        snap = mprofile.take_snapshot()
        mprofile.stop()

        # Snapshot references should remain valid after stopping profiler.
        found_it = False
        for trace in snap.traces:
            for frame in trace.traceback:
                if frame.name == "test_profile":
                    found_it = True
        self.assertTrue(found_it)

    def test_profile_concurrent(self):
        mprofile.start()
        t1 = threading.Thread(target=alloc_objects)
        t1.start()
        time.sleep(0.001)
        t2 = threading.Thread(target=alloc_objects)
        t2.start()

        time.sleep(0.001)
        snap = mprofile.take_snapshot()
        t1.join()
        mprofile.stop()
        t2.join()

    def test_profile_1frame(self):
        mprofile.start(max_frames=1)
        alloc_obj = object()
        snap = mprofile.take_snapshot()
        mprofile.stop()

        self.assertEqual(len(snap.traces), 1)
        self.assertEqual(len(snap.traces[0].traceback), 1)

    def test_profile_sampler(self):
        n = 100000
        mprofile.start(sample_rate=1024)
        alloc_obj = [object() for _ in range(n)]
        snap = mprofile.take_snapshot()
        mprofile.stop()

        self.assertGreaterEqual(len(snap.traces), 0)
        self.assertLessEqual(len(snap.traces), n)
        self.assertGreaterEqual(len(snap.traces[0].traceback), 2)

        stats = snap.statistics("traceback")
        self.assertGreaterEqual(len(stats), 1)
        self.assertGreaterEqual(stats[0].count, len(snap.traces))
        self.assertGreaterEqual(stats[0].size, sum(t.size for t in snap.traces))

if __name__ == '__main__':
    unittest.main()



from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import ctypes
import fnmatch
import glob
import io
import os
import re
import sys

from setuptools import Extension, setup

README = os.path.join(os.path.abspath(os.path.dirname(__file__)), "README.md")
with io.open(README, encoding="utf-8") as f:
    long_description = f.read()

def globex(pattern, exclude=[]):
    return [fn for fn in glob.iglob(pattern)
            if not any(fnmatch.fnmatch(fn, pattern) for pattern in exclude)]


pythonapi = ctypes.cdll.LoadLibrary(None)
if not hasattr(pythonapi, 'PyMem_SetAllocator'):
    print(("WARNING: PyMem_SetAllocatorEx: missing, %s has not been patched. " +
           "Heap profiler will attempt runtime patching") % sys.executable)


ext = Extension(
    "mprofile._profiler",
    language="c++",
    sources=globex("src/*.cc", exclude=["*_test.cc", "*_bench.cc"]) + [
        "third_party/google/tcmalloc/sampler.cc",
    ],
    depends=glob.glob("src/*.h"),
    include_dirs=[os.getcwd(), "src"],
    define_macros=[("PY_SSIZE_T_CLEAN", None)],
    extra_compile_args=["-std=c++11"],
    extra_link_args=["-std=c++11", "-static-libstdc++"],
)


def get_version():
  """Read the version from __init__.py."""

  with open("mprofile/__init__.py") as fp:
    # Do not handle exceptions from open() so setup will fail when it cannot
    # open the file
    line = fp.read()
    version = re.search(r"^__version__ = '([0-9]+\.[0-9]+(\.[0-9]+)?-?.*)'",
                        line, re.M)
    if version:
      return version.group(1)

  raise RuntimeError(
      "Cannot determine version from mprofile/__init__.py.")


setup(
    name="mprofile",
    version=get_version(),
    description="A low-overhead memory profiler.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    platforms=["Mac OS X", "POSIX"],
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)",
        "Operating System :: POSIX",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: Implementation :: CPython",
        "Topic :: Software Development :: Testing",
        "Topic :: Software Development :: Libraries :: Python Modules"
    ],
    project_urls={
        "Source": "https://github.com/timpalpant/mprofile",
        "Tracker": "https://github.com/timpalpant/mprofile/issues",
    },
    keywords="profiling performance",
    url="http://github.com/timpalpant/mprofile",
    author="Timothy Palpant",
    author_email="tim@palpant.us",
    license="MIT",
    setup_requires=["wheel"],
    packages=["mprofile"],
    ext_modules=[ext],
    test_suite="test",
)
